/*
	Copyright 2016, 2017 Michele "King_DuckZ" Santullo

	This file is part of MyCurry.

	MyCurry is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	MyCurry is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with MyCurry.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "vector.hpp"
#include "observersmanager.hpp"

namespace curry {
	class WorldGrid;

	class WorldSizeNotifiable {
	public:
		class DeferredRegister {
		public:
			DeferredRegister (WorldGrid* parWorld, WorldSizeNotifiable* parObj) noexcept;
			~DeferredRegister();
			WorldGrid* world() const { return m_world; }
		private:
			WorldSizeNotifiable* const m_notifiable;
			WorldGrid* const m_world;
		};

		virtual void world_size_changed (const vec2us& parSize, const vec2i& parPixelSize) = 0;

	protected:
		explicit WorldSizeNotifiable (const DeferredRegister& parDeferredRegister);
		virtual ~WorldSizeNotifiable() noexcept;
		void register_to_world (WorldGrid* parWorld);

	private:
		cloonel::ObserversManager<WorldSizeNotifiable*>::TicketType m_ticket;
		WorldGrid* m_world;
	};
} //namespace curry
