/*
	Copyright 2016, 2017 Michele "King_DuckZ" Santullo

	This file is part of MyCurry.

	MyCurry is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	MyCurry is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with MyCurry.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "vector.hpp"
#include "tileindextype.hpp"
#include "tileiterator.hpp"
#include "constrained_position.hpp"
#include "worldsizenotifiable.hpp"

namespace curry {
	class WorldGrid;

	class WorldViewport : public WorldSizeNotifiable {
	public:
		typedef TileIterator iterator;

		WorldViewport (WorldGrid* parWorld, vec2f parSize, const WorldSizeNotifiable::DeferredRegister& parDeferredRegister);
		virtual ~WorldViewport() noexcept = default;

		const WorldGrid* world() const;
		const vec2f& position() const;
		const vec2f& size() const;
		const TileIndex* tile (const vec2us& parIndex) const;
		void allow_overscan (bool parAllow);

		void set_position (const vec2f& parPos);
		virtual void world_size_changed (const vec2us& parSize, const vec2i& parPixelSize) override;

		iterator begin();
		iterator end();

	private:
		ConstrainedPosition<vec2f> m_position;
		vec2f m_size;
		WorldGrid* m_world;
	};
} //namespace curry
